from logging import Logger
import traceback
from typing import Any, Dict
from boto3 import client


def partial_send_sqs(url, sqs_client=None, **kw):
    if sqs_client is None:
        sqs = client("sqs")

    def send(payload: str, **kws):
        return sqs.send_message(QueueUrl=url, MessageBody=payload, **{**kw, **kws})

    return send


sfn = client("stepfunctions")
TASK_TOKEN = "taskToken"


def send_task_heartbeat(sID: str, out: Dict[str, Any], logger: Logger):
    if TASK_TOKEN in out:
        return logger.info(
            "send task heartbeat",
            sID=sID,
            token=out[TASK_TOKEN],
            res=sfn.send_task_heartbeat(taskToken=out[TASK_TOKEN]),
        )


def send_task_success(sID: str, result, out: Dict[str, Any], logger: Logger):
    if TASK_TOKEN in out:
        return logger.info(
            "send task success",
            sID=sID,
            token=out[TASK_TOKEN],
            res=sfn.send_task_success(taskToken=out[TASK_TOKEN], output=result),
        )


def send_task_failure(
    sID: str, exception: Exception, out: Dict[str, Any], logger: Logger
):
    if TASK_TOKEN in out:
        return logger.info(
            "send task failure",
            sID=sID,
            token=out[TASK_TOKEN],
            res=sfn.send_task_failure(
                taskToken=out[TASK_TOKEN],
                error=str(exception),
                cause=traceback.format_exc(),
            ),
        )
