import logging
from os import environ
from sys import stdout
from typing import Union
from loguru import logger

from functools import partialmethod

try:
    from orjson import dumps
except:
    from json import dumps


def orjson_log_sink(msg):
    r = msg.record
    rec = {
        "elapsed": r["elapsed"].total_seconds(),
        "time": r["time"].isoformat(),
        "level": {"name": r["level"].name, "no": r["level"].no,},
        "process": {"id": r["process"].id, "name": r["process"].name},
        "thread": {"id": r["thread"].id, "name": r["thread"].name},
        "file": r["file"].path,
    }

    if r["exception"]:
        rec["exception"] = str(msg)

    for k, v in r.items():
        print(k, type(v), v)
        if k in rec:
            continue
        rec[k] = v

    print(dumps(rec), flush=True)


try:
    BILL = logging.INFO + 1
    logger.level("BILL", BILL, color="<green>")
    logger.__class__.bill = partialmethod(logger.__class__.log, "BILL")
except:
    logger.exception("unable to set log level BILL = {}", BILL)

try:
    PERF = logging.DEBUG + 1
    logger.level("PERFORMANCE", PERF, color="<white>")
    logger.__class__.perf = partialmethod(logger.__class__.log, "PERFORMANCE")
except:
    logger.exception("unable to set log level PERFORMANCE = {}", PERF)


def json_log_config(
    log_level: Union[str, int] = logging.DEBUG,
    json: bool = not environ.get("DEBUG", False),
):
    if isinstance(log_level, int) or (
        isinstance(log_level, str) and (log_level in logging._nameToLevel)
    ):
        logging.root.setLevel(log_level)
    else:
        logging.root.setLevel(logging.DEBUG)

    # logging.basicConfig(handlers=[intercept_handler], level=LOG_LEVEL)
    # logging.root.handlers = [intercept_handler]

    if json:
        logger.configure(
            handlers=[
                {
                    "sink": orjson_log_sink,
                    "serialize": json,
                    "diagnose": True,
                    "backtrace": True,
                }
            ]
        )
    else:
        logger.configure(
            handlers=[
                {
                    "sink": stdout,
                    "serialize": False,
                    "format": "<green>{time}</green> | <level>{level: <8}</level> | <cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level> | <level>{extra}</level>",
                    "diagnose": True,
                    "backtrace": True,
                    "color": True,
                }
            ]
        )

    return logger
