from os import environ
from dotenv import load_dotenv
from threading import Timer
from io import BytesIO, TextIOWrapper


class Slots:
    @property
    def __dict__(self):
        return {s: getattr(self, s) for s in self.__slots__ if hasattr(self, s)}


def parse(cls, val, default=None):
    try:
        return cls(val)
    except:
        return default


def load_env_s3(
    s3bucket: str = environ.get("ENV_S3_BUCKET"),
    s3path: str = environ.get("ENV_S3_KEY"),
    refresh: float = environ.get("ENV_S3_REFRESH", 0),
    s3client=None,
    *arg,
    **kw
):
    if not (("ENV_S3_BUCKET" in environ) or ("ENV_S3_KEY" in environ)):
        return False
        
    if s3client is None:
        from boto3 import resource

        s3client = resource("s3")

    if refresh > 0:
        t = Timer(
            interval=refresh,
            function=load_env_s3,
            args=[s3bucket, s3path, refresh, s3client] + list(arg),
            kwargs=kw,
        )
        t.start()

    with BytesIO() as fp:
        s3client.Object(s3bucket, s3path).download_fileobj(fp)
        fp.seek(0)
        return load_dotenv(stream=TextIOWrapper(fp, "utf-8"), *arg, **kw)
